g = love.graphics -- so I don't have to type love.graphics

Class = require('libs/humb/class')

require('map')
require('player')

WINDOW_WIDTH, WINDOW_HEIGHT = 1280, 720

bg = {}
ground = {}

local difficulty = 1
local time = 0

local FONT_COLOR = {187/255,128/255,68/255} -- brown color


function love.load()

  -- setup window
  love.window.setMode(WINDOW_WIDTH, WINDOW_HEIGHT)

  -- fonts
  bigMenuFont = g.newFont('assets/fonts/ricasso/Ricasso-Regular.ttf', 45)
  smallMenuFont = g.newFont('assets/fonts/ricasso/Ricasso-Regular.ttf', 25)
  scoreFont = g.newFont('assets/fonts/ricasso/Ricasso-Regular.ttf', 32)

  -- images
  bg.img = g.newImage('assets/graphics/BG/Background1.png')
  ground.img = g.newImage('assets/graphics/BG/FG.png')

  gameState = 'menu'

  -- load the game
  prepareGame()

end


function love.update(dt)

  -- if we are not paused and we are in the play state then update the game
  if gameState ~= 'pause' then
    if gameState == 'play' then

      -- update x value of the bg and fg so it loops around
      bg.x = (bg.x + bg.scrollingSpeed * dt) % (bg.w/2)
      ground.x = (ground.x + ground.scrollingSpeed * dt) % (ground.w/2)

      -- increase time and if we hit 20 second increase the speed of the game
      time = time + dt
      if time >= 20 and difficulty <= 1.8 then
        difficulty = difficulty + 0.1

        -- update the speed of the game and reset timer
        ground.scrollingSpeed = ground.baseScrollingSpeed * difficulty
        time = 0
      end

      map:update(dt)
      player:update(dt)

      -- wait only a little bit more before resetting
      -- so if player presses space or s once they land or when the slide
      -- ends then they can jump or slide again
      -- this makes the game more responsive
      keysTimer = keysTimer + dt
      if keysTimer >= 0.01 then
          keysPressed = {}
          keysTimer = 0
      end
    end
  end

end


function love.draw()

  -- draw background
  g.draw(bg.img, -bg.x, 0)
  g.draw(map.spriteSheet, map.sprites[GOLD_TROPHY], 0, 0, nil, 1.5, 1.5)
  g.setFont(scoreFont)
  g.setColor(unpack(FONT_COLOR))
  g.print(player.score, 100, 25)
  g.setColor(1,1,1)

  -- playing or pausing?
  if gameState == 'play' or gameState == 'pause' then

    -- draw ground
    g.draw(ground.img, -ground.x, ground.y)

    map:render()
    player:render()

    -- if paused, then draw the pause message
    if gameState == 'pause' then
      drawPauseScreen()
    end
  end

  -- in menu? draw the menu
  if gameState == 'menu' then
    drawMenuScreen()
  end

  -- game finished? draw the game finished screen
  if gameState == 'finish' then
    drawFinishScreen()
  end

end


function love.keypressed(key)

  if key == 'escape' then love.event.push('quit') end

  -- add the key pressed to the keysPressed table
  keysPressed[key] = true

  -- pause and un-pause game
  if key == 'p' and gameState == 'play' then
    gameState = 'pause'
  elseif gameState == 'pause' then
    gameState = 'play'
  end

  -- start the game when enter or return is hit
  if gameState == 'menu' and key == 'return' then
    gameState = 'play'
  end

  -- restart the game if game is finished
  if gameState == 'finish' and key == 'r' then
      prepareGame()
       gameState = 'play'
  end

end


-- to enquire if a key was pressed
function keyWasPressed(key)
  return keysPressed[key] -- return true if the key exists in the table
end


-- reset game to its initial values
function prepareGame()

  difficulty = 1
  keysPressed = {}  -- to store the keys pressed so we can make the jump in the player class
  keysTimer = 0 -- to improve responsive, see love.update()
  time = 0

  -- bg image
  bg.w = bg.img:getWidth()
  bg.h = bg.img:getHeight()
  bg.scrollingSpeed = 200
  bg.x = 0

  -- ground
  ground.w = ground.img:getWidth()
  ground.h = ground.img:getHeight()
  ground.scrollingSpeed = 600  -- a little bit faster than background so it can feel realistic
  ground.baseScrollingSpeed = ground.scrollingSpeed -- we can multiply this to increase difficulty
  ground.x = 0
  ground.y = WINDOW_HEIGHT - ground.h

  -- initiate map and player
  map = Map()
  player = Player()
end


-- draws the pause screen
function drawPauseScreen()
    g.setFont(bigMenuFont)
    g.setColor(unpack(FONT_COLOR))
    g.printf('Press any key to continue!', 0, WINDOW_HEIGHT/2, WINDOW_WIDTH, 'center')
    g.setColor(1,1,1)
end


-- draws the menu screen
function drawMenuScreen()
  g.clear(0,0,0)
  g.setFont(bigMenuFont)
  g.setColor(1, 1, 1)
  g.printf('Press Enter to Start!', 0, WINDOW_HEIGHT/3, WINDOW_WIDTH, 'center')
  g.setFont(smallMenuFont)
  g.printf('Controls: ', 0, WINDOW_HEIGHT/2, WINDOW_WIDTH/2, 'center')
  g.printf('space:        Jump', 350, (WINDOW_HEIGHT/2 + 50), WINDOW_WIDTH/2, 'left')
  g.printf('s:               Slide', 350, (WINDOW_HEIGHT/2 + 100), WINDOW_WIDTH/2, 'left')
  g.printf('p:               pause', 350, (WINDOW_HEIGHT/2 + 150), WINDOW_WIDTH/2, 'left')
end


-- draws the finish screen
function drawFinishScreen()
  g.setFont(bigMenuFont)
  g.setColor(unpack(FONT_COLOR))
  g.printf('Press R to replay!, or press Esc to exit!', 0, WINDOW_HEIGHT/2, WINDOW_WIDTH, 'center')
  g.setColor(1,1,1)
end