require('Util')

Map = Class{}

MAP_TILE_WIDTH, MAP_TILE_HEIGHT = 64, 64

-- tiles numbers
local TRAPS = {71, 72, 43}
local TROPHIES = {50, 53, 52, 51} -- we put the gold trophy last so we can exclude it when randomizing
local timerSpeed = 3

-- we have to store the tiles by its own so we can calculate the score and manipulate the y position if needed
BRICK_TRAP, FORK_TRAP, FAN_TRAP = 43, 71, 72
BLUE_TROPHY, GOLD_TROPHY, GREEN_TROPHY, RED_TROPHY = 50, 51, 52, 53

local spawnTimer  -- to randomize the spawning of obstacles



function Map:init()

  self.spriteSheet = g.newImage('assets/graphics/tileSheet/platformPack_tilesheet.png')
  self.sprites = generateQuads(self.spriteSheet, MAP_TILE_WIDTH, MAP_TILE_HEIGHT)
  self.obstacles = {}

  spawnTimer = 300

end


function Map:update(dt)

  -- decrease timer and create obstacle when we hit 0
  spawnTimer = spawnTimer - timerSpeed
  if spawnTimer <= 0 then

    -- reset the timer again to random value
    spawnTimer = love.math.random(140, 300)
    self:spawnObstacles()
  end

  -- remove obstacles that gone out of screen
  for i = #self.obstacles, 1, -1 do
    local o = self.obstacles[i]
    if o.x < -MAP_TILE_WIDTH then table.remove(self.obstacles, i) end
  end

  -- update position of obstacles
  for _, o in ipairs(self.obstacles) do
      o.x = o.x - ground.scrollingSpeed * dt
  end
end


function Map:render()

  -- loop obstacles
  for _, o in ipairs(self.obstacles) do

    -- to be used in the draw function if object need rotation
    local rotation = 1
    local xOffset = nil

    -- if it is the brick set rotation value
    if o.id == BRICK_TRAP then
      rotation, xOffset = -1, MAP_TILE_HEIGHT
    end

    -- then draw
    g.draw(self.spriteSheet, self.sprites[o.id], o.x, o.y, nil, 1, rotation, nil, xOffset)
  end
end


-- randomly spawn obstacles, keys and doors
function Map:spawnObstacles()

  -- this will create a chance of 60% obstacle, 25% stars, 10% doors, 5% keys
  toss = love.math.random(1, 100)

  local obstacle = {}
  obstacle.x = WINDOW_WIDTH
  local oPosition = love.math.random(2) == 1 and 1 or 2.5  -- we can multiply by this number to get the y value

  -- in 1-60 range? pick a random obstacle and add it to the table
  if toss >= 1 and toss <= 60 then
    obstacle.id = TRAPS[love.math.random(#TRAPS)]

    -- if the trap is fork then it must be in the ground so we reset the position value to 1
    if obstacle.id == FORK_TRAP then oPosition = 1 end

    obstacle.y = WINDOW_HEIGHT - ground.h - (MAP_TILE_HEIGHT * oPosition) -- this to put up or down
    table.insert(self.obstacles, obstacle)


    -- 30% of trophies
    if love.math.random(1, 100) <= 30 then
      local trophy = {}
      trophy.x = WINDOW_WIDTH
      local tPosition = oPosition == 1 and 2.5 or 1  -- opposite of the obstacle position

      trophy.y = WINDOW_HEIGHT - ground.h - (MAP_TILE_HEIGHT * tPosition)

      -- 10% gold trophy, 80% others
      local trophyIdToss = love.math.random(1, 100)
      if trophyIdToss <= 10 then
        trophy.id = TROPHIES[4]
      else
        trophy.id = TROPHIES[love.math.random(3)]
      end
      table.insert(self.obstacles, trophy)
    end
  end

end
